/*******************************
	Sandstorm
	
	MAIN.H
********************************/

/**** DEFINES ****/

// game related
// INTERLACED mode flag
//#define INTER

//resolution
#ifdef INTER

#define WIDTH_NTSC	(640)    //for NTSC TVs
#define HEIGHT_NTSC	(480)	 //for NTSC TVs
#define WIDTH_PAL	(640)	 //for PAL TVs
#define HEIGHT_PAL	(512)	 //for PAL TVs

#endif

#ifndef INTER

#define WIDTH_NTSC	(320)    //for NTSC TVs
#define HEIGHT_NTSC	(240)	 //for NTSC TVs
#define WIDTH_PAL	(320)	 //for PAL TVs
#define HEIGHT_PAL	(256)	 //for PAL TVs

#endif

//graphics related
#define OTLENGTH	(8)		//length of OT (8)
#define PROJ		(1<<9)		//projection distance
#define PACKETMAX	(10000*24)
#define NUMCARS		(3)		//num of cars including yourself
#define BGWIDTH		(320)		//total width of both sky sprites
#define MAXITEMS	(5)		//max. num. of items at a given time
#define BULL_SPEED	(35)		//bullet speed when fired
#define MISSILE_SPEED	(30)		//missile speed when fired
#define BULL_GAP	(10)		//firing gap to slow down firing pace
#define FREEZETIME	(350)		//amount of time CPU players "freeze" when time item is used
#define SMOOTHPAUSE	(10)		//smoothes item display in ChangeItem()
#define SLIDESPEED	(5)		//how fast the vehicle slides when hit
#define FIRE_DAMAGE	(6)		//damage a fire missile causes
#define HOMING_DAMAGE	(4)		//homing missile damage
#define POWER_DAMAGE	(8)		//power damage
#define BULLET_DAMAGE	(1)	
#define RICO_DAMAGE	(5)
#define MINE_DAMAGE	(10)
#define MINE_SLIDE	(10)		//how fast the vehicle slides when hit with a mine		
#define HEALTH_PLUS	(40)		//amount of life bar width gained from health item
#define COLL_DAMAGE	(3)		//damage when hit by another car
#define MVOL		(127)		//main volume level
#define SVOL		(127)		//seq volume level
#define DFILE 		(2)             // sound related files to load from CD

//memory addresses
#define VH_ADDR 0x80090000 /* 60448 */
#define VB_ADDR 0x8009ec20 /* 401360 */
#define SEQ_ONE_ADDR 0x80100bf0 /* 48508 */
#define SAND_DATA 0x8010c970 /* 11336 */
#define SANDTEX_DATA 0x8010f5c0 /* 8272 */
#define BIRTCH_DATA 0x80111610 /* 13382 */
#define LEFTSKY_DATA 0x80114a60 /* 38944 */
#define RIGHTSKY_DATA 0x8011e280 /* 38944 */
#define TRUCKW_DATA 0x80127aa0 /* 3004 */
#define TRUCKR_DATA 0x80128660 /* 3004 */
#define TRUCKG_DATA 0x80129220 /* 3004 */
#define PYRAMID_DATA 0x80129de0 /* 1480 */
#define SUN2_DATA 0x8012a3b0 /* 2832 */
#define MEGA_DATA 0x8012aec0 /* 17872 */
#define ONSCREEN_DATA 0x8012f490 /* 13540 */
#define POINTER_DATA 0x80132980 /* 1456 */
#define POWER_M_DATA 0x80132f30 /* 408 */
#define HOMING_M_DATA 0x801330d0 /* 408 */
#define FIRE_M_DATA 0x80133270 /* 408 */


/**** GLOBALS ****/

//graphics related
extern int out_buf;
extern GsOT wot[2], wot2[2];						//OT headers
extern GsOT_TAG ot[2][1<<OTLENGTH], ot2[2][1<<OTLENGTH];		//actual OTs
extern PACKET gpu_work_area[2][PACKETMAX];	//GPU Packet work area
extern u_char mode;				//checking for video mode

//camera and light related
extern GsRVIEW2 view;
extern GsF_LIGHT light[3];
extern GsFOGPARAM fog;

//sprite instances
extern GsSPRITE tree[5], leftbg, rightbg, sun2, upg_ico, mine_ico, fire_ico, hlt_ico,
	hom_ico, mine_ico, pow_ico, rico_ico, tbo_ico, time_ico, bullet[21], mine_1, mine_2,
	mine_3, ricochet, os_hom, os_fire, os_pow, os_rico, os_mine, os_bullet, os_time, fire1, fire2,
	fire3a, fire3b, fire4a, fire4b, fire5a, fire5b;
extern GsBOXF le1, le2, le3, life, turbo, cpulife1, cpulife2, radar, blip, blip2, dot, blank;

extern GsCOORDINATE2 oldcoord0, oldcoord1, oldcoord2;	//restoring car positions in colllision

//graphics related
extern GsCOORDINATE2 dir_coord, dir_coord2;   //control movement of the car 
extern SVECTOR dir_vec, dir_vec2;           //in the direction it faces

//object structures
struct object
{
	GsDOBJ2 obj;
	GsCOORDINATE2 coord;
	SVECTOR vector;
};

//structures
struct DB
 {

   DRAWENV draw;
   DISPENV disp; 
         
 };

//player inventory structure
struct inventory
{
	short upg, mine, fire, hlt, hom, pow, rico, tbo, time;
};

//bounding sphere info for each object 
struct sphere{
	short radius; 
};

//extern structure instance declarations
extern struct object sand, track, truck[NUMCARS], pyramid, sand2, pointer, homing_m, power_m,
	 fire_m, homing_m, power_m, fire_m, homing2_m, power2_m, fire2_m, homing3_m, power3_m, 
	 fire3_m;

extern struct DB buff[2];
extern struct inventory p1items;
extern struct sphere greencar, whitecar, redcar, item, missile, bulletsphere, mine_expl, car_coll;

//game related
extern int gameover, pause;
extern volatile u_char *bb0,*bb1;
extern u_long padd,padd2;
extern int gamemode, level, wincount;	//level is difficulty level; 0-easy; 1-normal; 2-hard
extern int debugmode;
extern short bull_stat[21];
extern int timelimit[9];	//used for item managment 
extern int rot;	//car rotation variable
extern int spot[9];		//locations of items
extern int bull_dir[21];	//directions for projectiles
extern long SIN[360];
extern long COS[360];
extern short sel_item, sel_item_old, lock_sel9;		//selected item
extern int x_speed, z_speed, rico_life;	//ricochet variables
extern int rico_status, rico_dir;
extern int mine_status, mine_dir;
extern int picksprite;
extern int fire_m_status, homing_m_status, power_m_status, fire_m_dir, homing_m_dir, power_m_dir,
	fire2_m_status, homing2_m_status, power2_m_status, fire2_m_dir, homing2_m_dir, power2_m_dir,
	fire3_m_status, homing3_m_status, power3_m_status, fire3_m_dir, homing3_m_dir, power3_m_dir, firestatus;

extern int freeze, actcounter1, actcounter2, actcounter3, actcounter4, actcounter5,  actcounter6,
	expl_status, expl_status2, expl_status3, detonate;	
extern int p1destroyed, cpu1destroyed, cpu2destroyed; 

//2d object pos in world
extern VECTOR pos[5], sunpos, upg_ico_pos, mine_ico_pos, fire_ico_pos, hlt_ico_pos,
	hom_ico_pos, mine_ico_pos, pow_ico_pos, rico_ico_pos, tbo_ico_pos, time_ico_pos, 
	bullet_pos[21], mine_1_pos, mine_2_pos, mine_3_pos, rico_pos, fire1_pos, fire2_pos;


//AI attack state control
extern int fired1_p, fired1_f, fired1_h, homing2_status, homing2_count, fire2_h_count, fire2_h_status;
extern int fired2_p, fired2_f, fired2_h, homing3_status, homing3_count, fire3_h_count, fire3_h_status;
